﻿using FLY.Thick.FilmCasting.IService;
using MultiLayout.UiModule;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Unity;

namespace FLY.Thick.FilmCasting.UI.UiModule
{
    /// <summary>
    /// DynAreaProfile.xaml 的交互逻辑
    /// </summary>
    public partial class DynAreaProfile : UserControl
    {
        IUnityContainer container;
        IFilmCastingProfileService profileService;
        IBulkDbService bulkDbService;
        public DynAreaProfile()
        {
            InitializeComponent();
        }

        [InjectionMethod]
        public void Init(
            IUnityContainer container,
            IFilmCastingProfileService profileService,
            IBulkDbService bulkDbService
            )
        {
            this.container = container;
            this.profileService = profileService;
            this.bulkDbService = bulkDbService;

            button_profile.DataContext = this.profileService.Param;
            button_bulkdb.DataContext = bulkDbService;
        }

        private void ButtonRollDown_Click(object sender, RoutedEventArgs e)
        {
            if (!bulkDbService.IsFinished)
            {
                bulkDbService.Finish();
            }
        }

        private void ButtonProfile_Click(object sender, RoutedEventArgs e)
        {
            PgProfile p = new PgProfile();
            container.BuildUp(p);
            MultiLayout.FlyLayoutManager.NavigationService.Navigate(p);
        }
    }

    public class DynAreaProfileVmUt : INotifyPropertyChanged
    {
        /// <summary>
        /// 产品名称
        /// </summary>
        public string PName { get; set; } = "product name";
        /// <summary>
        /// 批号
        /// </summary>
        public string Batch { get; set; } = "batch";
        /// <summary>
        /// 卷号
        /// </summary>
        public string Number { get; set; } = "1";

        public double Target { get; set; } = 100.2;
        /// <summary>
        /// 斜率K
        /// </summary>
        public double K { get; set; } = 1;

        /// <summary>
        /// 开始产生数据时间
        /// </summary>
        public DateTime StartTime { get; set; }

        /// <summary>
        /// 最后产生数据时间
        /// </summary>
        public DateTime EndTime { get; set; }

        /// <summary>
        /// 当前生产，已经结束，已经下辊了
        /// </summary>
        public bool IsFinished { get; set; }


        public event PropertyChangedEventHandler PropertyChanged;
    }

    public class UiModule2_DynAreaProfile : IUiModule2
    {
        /// 控件标题
        /// 它的值取决于culture
        /// </summary>
        public string Title => "测厚.产品状态";

        public ComponentType Type => ComponentType.DynArea;

        public bool IsUnique => true;

        /// <summary>
        /// 控件
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetComponent(int id, IUnityContainer container)
        {
            DynAreaProfile graph = new DynAreaProfile();
            container.BuildUp(graph);
            return graph;
        }

        /// <summary>
        /// 控件缩略图，用于编辑界面时，大致看看
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetThumbnail()
        {
            return new System.Windows.Controls.Grid();
        }


        /// <summary>
        /// 给出全部控件ID, 控件自行删除没有的参数
        /// </summary>
        /// <param name="IDs"></param>
        public void MatchParam(int[] IDs)
        {

        }
    }
}
